#
# $Id: nnrpd_auth.pl.in,v 1.1.2.1 1999/01/05 22:29:59 kondou Exp $
#
# Lokma nnrp authentication. Derivated from inn sample code (nnrpd_auth.pl)
#
# $Log$
#
# This file is loaded when nnrpd starts up. If it defines a sub named
# `authenticate', then that function will be called during processing of a
# connect, auth request or disconnect.  Attributes about the connection are 
# passed to the program in the %attributes global variable.  It should return 
# an array with 4 elements:
#
# 1) NNTP response code.  Should be one of the codes from %connectcodes or %authcodes
# 2) Reading Allowed. Should be a boolean value.
# 3) Posting Allowed. Should be a boolean value.
# 4) Wildmat expression that says what groups to provide access to.
#
# All four of these are required.  If there is a problem with them then nnrpd
# will die and syslog the exact reason.

$dbdir = "/etc/lokmadb";

require "/usr/lib/innshellvars.pl";

my (%connectcodes) = ("read/post" => 200, "read" => 201, "authneeded" => 480, "permdenied" => 502);
my (%authcodes)    = ("allowed" => 281, "denied" => 502);

# This is called by nnrpd when it first starts up.
sub auth_init {
	&db_init;
}

# This is called when a user connects or authenticates
sub authenticate {
  my $key;
  foreach $key (keys %attributes) {
  }
  if ($attributes{type} eq "connect") {
	return (&permtocode, undef, undef, undef);
  } elsif ($attributes{type} eq "authenticate") {
    return checkuser();
  } elsif ($attributes{type} eq "disconnect") {
  }
  return 502;
}

sub checkuser {
  my $user = $attributes{'username'};
  my $pass = $attributes{'password'};

  $user =~ tr/A-Z/a-z/;
  $pass =~ tr/A-Z/a-z/;
  $_ = open(IN, "$dbdir/user/$user");
  if (!$_) {
    return ($authcodes{'denied'}, undef, undef, undef);
  }
  while(<IN>) {
	if (/^password: (.*)$/i) {
		$pw = $1;
		$pw =~ tr/A-Z/a-z/;
	}
	elsif (/^groups: (.*)$/) {
		$groups = $1;
	}
  }
  close IN;

	# there may be more than one password, separated by |.
	@pws = split(/\|/, $pw);
	$ok = 0;
	foreach(@pws) {
		$_ =~ tr/A-Z/a-z/;
		if ($_ eq $pass) {
			$ok = 1;
		}
	}
	return ($authcodes{'denied'}, undef, undef, undef) if ($ok != 1);
	# pw ok, get newsgroups
	@subscr = ();
	@groups = split(/\s+/, $groups);
	foreach $group (@groups) {
		open(IN, "$dbdir/group/$group") || next;
		while(<IN>) {
			if (/^newsgroups:\s+(.*)$/) {
				push(@subscr, split(/\s+/, $1));
			}
		}
		close IN;
	}
	$subscr = "";
	$subscr = shift(@subscr) if (@subscr);
	foreach(@subscr) {
		$subscr .= "," . $_;
	}
	return ($authcodes{'allowed'}, 1, 1, $subscr);
}

sub permtocode {
  my ($read, $post) = @_;

  return $connectcodes{'read/post'} if ($post);
  return $connectcodes{'read'} if ($read);
  return $connectcodes{'authneeded'};
}

sub db_init {
}
